include "Vram.inc"
include "Chars.inc"

ext WRTVRM, RDVRM, LDIRVM, FILVRM
ext pStage_, StageMap_, topRow_, yMod_
ext DrawBlocks_
ext dotOffset_, charOffset_
ext BackgroundPattern_
ext VRas

VisibleFloorCount equ (WindowHeight+FloorHeight-1)/FloorHeight+1
FloorHeight equ 4
ColumnCount equ 12
MapWidth equ ColumnCount/4

CellType_Space equ 0
CellType_Ladder equ 1
CellType_Wall equ 2
CellType_Hole equ 3

Blanking equ 8*(3+2)

; void InitVdp();
cseg
VdpValues:
    defb 56h,0e2h
    defb 0ffh,0ffh,0ffh,0ffh,0fbh,00h
    defb 00h,00h,0ffh
ColorValues:
    defb 0,0
    defb 0,15
    defb 15,0
    defb 15,15
    defb 240,0
    defb 240,15
    defb 255,0
    defb 255,15
    defb 0,0
    defb 0,12
    defb 12,0
    defb 12,12
    defb 192,0
    defb 192,12
    defb 124,0
    defb 100,12
InitVdp_: public InitVdp_
    push af | push hl | push de | push bc
        ld hl,VdpValues
        ld c,0
        ld b,11
        do
            ld a,(hl) | out (VdpPort+1),a
            inc hl
            ld a,c | or 80h | out (VdpPort+1),a
            inc c
        dwnz

        xor a | out (VdpPort+1),a
        ld a,0c0h | out (VdpPort+1),a
        ld c,2
        do
            ld hl,ColorValues
            ld b,16*2
            do
                ld a,(hl) | inc hl
                out (VdpPort),a
            dwnz
            dec c
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret

; void InitMonoChar(word vram, ptr<byte> p, byte count, byte color);
dseg
InitMonoChar_@Param3: public InitMonoChar_@Param3
    defb 0
cseg
InitMonoChar_: public InitMonoChar_
    push af | push hl | push de | push bc
        ld a,l | out (VdpPort+1),a
        ld a,h | or 40h | out (VdpPort+1),a
        ex de,hl
        do
            push bc
                ld c,CharHeight
                do
                    ld e,(hl) | inc hl
                    ld a,(InitMonoChar_@Param3)
                    ld d,a
                    ld b,4
                    do
                        bit 0,d
                        if nz
                            ld a,e
                        else
                            xor a
                        endif
                        out (VdpPort),a
                        srl d
                    dwnz
                    dec c
                while nz | wend
            pop bc
            dec c
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret


; void InitColorChar(word vram, ptr<byte> p, byte count);
cseg
InitColorChar_: public InitColorChar_
    push af | push hl | push de | push bc
        ld a,l | out (VdpPort+1),a
        ld a,h | or 40h | out (VdpPort+1),a
        ex de,hl
        do
            ld b,CharHeight*4
            do
                ld a,(hl) | inc hl
                out (VdpPort),a
            dwnz
            dec c
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret


; void ClearScreen();
cseg
ClearScreen_:
    public ClearScreen_
    push af | push hl | push bc
        ld hl,Vram_PatternName
        ld bc,VramHeight*VramRowSize
        xor a
        call FILVRM

        ld hl,Vram_SpriteAttribute
        ld bc,64
        ld a,0c8h
        call FILVRM
    pop bc | pop hl | pop af
ret


; word VramAddress(byte x, byte y);
VramAddress_: public VramAddress_
    push de
        ld l,e
        ld h,0
        add hl,hl ;*2
        add hl,hl ;*4
        add hl,hl ;*8
        add hl,hl ;*16
        add hl,hl ;*32
        add hl,hl ;*64
        add a,a
        ld e,a
        ld d,0
        add hl,de
        ld de,Vram
        add hl,de
    pop de
ret


; word Put(word vram, byte c);
cseg
Put_: public Put_
    push af
        di
            ld a,l | out (VdpPort+1),a
            ld a,h | or 40h | out (VdpPort+1),a
            ld a,e
            out (VdpPort),a
            nop | nop | nop
            xor a
            out (VdpPort),a
        ei
    pop af
    inc hl
    inc hl
ret


; word PrintC(word vram, byte c);
cseg
PrintC_: public PrintC_
    ex af,af'
        ld a,e
        sub ' '
        call WRTVRM
    ex af,af'
    inc hl
    inc hl
ret


; void UpdateBackgroundChars();
cseg
UpdateBackgroundChars_: public UpdateBackgroundChars_
    push af | push hl | push de | push bc
        do
            ld a,(VRas)
            cp Blanking
        while c | wend

        ld a,(charOffset_)
        or a
        if z
            ld hl,4000h+Vram_PatternGenerator+Char_Background*CharHeight*4
        else
            ld hl,4000h+Vram_PatternGenerator+Char_Alter*CharHeight*4
        endif

        ld a,(dotOffset_)
        ld d,a

        ld c,VdpPort
        di
            inc c
                out (c),l
                out (c),h
            dec c
            ld hl,BackgroundPattern_
            ld e,Char_Alter-Char_BackGround
            do
                push hl
                    ld a,7
                    sub d
                    add a,a ; *2
                    add a,a ; *4
                    add a,l
                    ld l,a
                    ld a,h
                    adc a,0
                    ld h,a
                    ld b,8*4
                    do
                        ld a,(hl)
                        inc hl
                        out (c),a
                    dwnz
                pop hl
                ld a,l
                add a,15*4
                ld l,a
                ld a,h
                adc a,0
                ld h,a
                dec e
            while nz | wend
        ei
    pop bc | pop de | pop hl | pop af
ret


; void MapToVram();
cseg
Repeat: public Repeat
    push af | push bc
        do
            ld a,(VRas)
            cp Blanking
        while nc | wend

        ld a,(charOffset_)
        ld c,a
        do
            exx
                ld a,e
                inc e
            exx
            cp WindowHeight
            if c
                di
                    ld a,l | out (VdpPort+1),a
                    ld a,h | or 40h | out (VdpPort+1),a
                    ld a,d
                    or a
                    if nz
                        add a,c
                    endif
                    out (VdpPort),a
                    push af | pop af
                    xor a | out (VdpPort),a
                    push af | pop af
                    ld a,e
                    or a
                    if nz
                        add a,c
                    endif
                    out (VdpPort),a
                    push af | pop af
                    xor a | out (VdpPort),a
                ei
            endif
            push de
                ld de,VramRowSize
                add hl,de
            pop de
        dwnz
    pop bc | pop af
ret

MapToVram_: public MapToVram_
    push af | push hl | push de | push bc | push ix | push iy
        do
            ld a,(VRas)
            cp Blanking
        while nc | wend

        ld iy,(pStage_)
        ld a,(topRow_)
        exx
            ld d,a ; row
        exx
        or a
        if p
            ld ix,StageMap_
            ld e,a
            add a,a ;*2
            add a,e ;*3
            ld e,a
            ld d,0
            add ix,de ; pMap
        else
            ld ix,StageMap_-MapWidth
        endif
        exx
            ld a,(yMod_)
            or a
            if nz
                neg
            endif
            ld e,a
        exx

        ld hl,VramTop+4000h
        ld de,-VramRowSize
        do
            or a
        while m
            add hl,de
            inc a
        wend
        ld c,VisibleFloorCount
        do
            ld b,MapWidth
            do
                push bc
                    exx
                        ld a,d ; row
                        or a
                        if p
                            if z
                                ld c,0 ; upper
                            else
                                ld c,(ix-MapWidth)
                            endif
                            ld h,(ix) ; middle
                            inc a
                            cp (iy+0)
                            if c
                                ld l,(ix+MapWidth) ; lower
                            else
                                ld l,0
                            endif
                        else
                            ld h,0 ; middle
                            ld l,(ix+MapWidth) ; lower
                            ld c,0ffh ; upper
                        endif
                    exx
                    inc ix
                    ld c,4
                    do
                        exx
                            ld a,h ; middle
                        exx
                        and 3
                        or a
                        if z
                            exx
                                ld a,c ; upper
                            exx
                            and 3
                            cp CellType_Hole
                            if z
                                ld de,(Char_Space shl 8) or Char_Space
                            else
                                ld de,(Char_Floor_Space shl 8) or Char_Floor_Space
                            endif
                            ld b,1
                            call Repeat
                            exx
                                ld a,l ; lower
                            exx
                            and 3
                            cp CellType_Ladder
                            if z
                                ld de,(Char_Space_LeftLadder shl 8) or Char_Space_RightLadder
                                ld b,1
                                call Repeat
                                ld de,(Char_LeftLadder_LeftLadder shl 8) or Char_RightLadder_RightLadder
                                ld b,2
                                call Repeat
                            else
                                cp CellType_Wall
                                if z
                                    ld de,(Char_Space shl 8) or Char_Space
                                    ld b,2
                                    call Repeat
                                    ld de,(Char_Space_Wall shl 8) or Char_Space_Floor
                                    ld b,1
                                    call Repeat
                                else
                                    ld de,(Char_Space shl 8) or Char_Space
                                    ld b,2
                                    call Repeat
                                    cp CellType_Wall
                                    if z
                                        ld de,(Char_Space_Wall shl 8) or Char_Space_Floor
                                    else
                                        ld de,(Char_Space_Floor shl 8) or Char_Space_Floor
                                    endif
                                    ld b,1
                                    call Repeat
                                endif
                            endif
                        else
                            cp CellType_Ladder
                            if z
                                ld de,(Char_LeftLadder_LeftLadder shl 8) or Char_RightLadder_RightLadder
                                ld b,3
                                call Repeat
                                exx
                                    ld a,l ; lower
                                exx
                                and 3
                                cp CellType_Ladder
                                if z
                                    ld de,(Char_LeftLadder_LeftLadder shl 8) or Char_RightLadder_RightLadder
                                else
                                    cp CellType_Wall
                                    if z
                                        ld de,(Char_LeftLadder_Wall shl 8) or Char_RightLadder_Floor
                                    else
                                        ld de,(Char_LeftLadder_Floor shl 8) or Char_RightLadder_Floor
                                    endif
                                endif
                                ld b,1
                                call Repeat
                            else
                                cp CellType_Wall
                                if z
                                    ld de,(Char_Wall_Wall shl 8) or Char_Floor_Space
                                    ld b,1
                                    call Repeat
                                    ld de,(Char_Wall_Wall shl 8) or Char_Space
                                    ld b,2
                                    call Repeat
                                    exx
                                        ld a,l ; lower
                                    exx
                                    and 3
                                    cp CellType_Wall
                                    if z
                                        ld de,(Char_Wall_Wall shl 8) or Char_Space_Floor
                                    else
                                        ld de,(Char_Wall_Floor shl 8) or Char_Space_Floor
                                    endif
                                    ld b,1
                                    call Repeat
                                else
                                    exx
                                        ld a,c ; upper
                                    exx
                                    and 3
                                    cp CellType_Hole
                                    if z
                                        ld de,(Char_Space shl 8) or Char_Space
                                    else
                                        ld de,(Char_Floor_Space shl 8) or Char_Floor_Space
                                    endif
                                    ld b,1
                                    call Repeat
                                    ld de,(Char_Space shl 8) or Char_Space
                                    ld b,3
                                    call Repeat
                                endif
                            endif
                        endif
                        ld de,-VramRowSize*4+2*VramStep
                        add hl,de
                        exx
                            srl h | srl h
                            srl l | srl l
                            srl c | srl c
                            dec e | dec e | dec e | dec e
                        exx
                        dec c
                    while nz | wend
                pop bc
            dwnz
            ld de,VramRowSize*4-2*VramStep*ColumnCount
            add hl,de
            exx
                inc d
                inc e | inc e | inc e | inc e
            exx
            dec c
        while nz | wend

        call DrawBlocks_
    pop iy | pop ix | pop bc | pop de | pop hl | pop af
ret
